export type SFn = () => string;
export type SimpleSymbol = {
    name: string;
    value?: string | SFn;
    parent?: SimpleSymbol;
    children: DumbSymTbl;
};
export type Sym = {
    name: string;
    value?: string | SFn;
    parent?: SymbolTable;
    children?: SymbolTable;
};
export type DumbSymTbl = Map<string, SimpleSymbol>;
export type SymbolTable = {
    add: (name: string | string[], value: string | SFn) => Sym;
    get: (lkup: string | string[]) => Sym;
    check: (lkup: string | string[]) => Sym | undefined;
    parent: () => Sym | undefined;
};
export type ParsedFile = {
    scopedTable: DumbSymTbl;
};
export type DependentValue = {
    value: string;
    unresolved: Set<string>;
};
export type FilterFunc = (str: SimpleSymbol) => boolean;
export type ValueMakerFunc = (vrbl: SimpleSymbol) => DependentValue;
export interface Condition {
    op: 'eq' | 'neq' | 'def' | 'ndef' | 'raw';
    variable: string;
    value?: string;
}
export type DependentUpon = {
    dependsOn: string[];
};
export type Definition = DependentUpon & {
    name: string;
    type: 'decl' | 'seq' | 'add' | '?decl';
    value: string;
    condition: Condition[];
};
export type ScopedName = {
    getElement: (index: number) => string;
    getFullName: () => string;
    length: () => number;
};
export type LibDependency = {
    name: string;
    ver?: string;
};
export type SemVer = string | {
    major: number;
    minor: number;
    patch: number;
};
export type Categories = 'Uncategorized' | 'Display' | 'Communication' | 'Signal Input/Output' | 'Sensors' | 'Device Control' | 'Timing' | 'Data Storage' | 'Data Processing' | 'Other';
export type OptionalLibProps = {
    version: SemVer;
    author: string[];
    maintainer: string;
    sentence: string;
    paragraph: string;
    category: Categories;
    url: string;
    architecture: string;
    depends: LibDependency[];
    staticLink: boolean;
    includes: string[];
    precompiled: boolean | 'full';
    ldflags: string;
};
export type LibProps = Partial<OptionalLibProps> & {
    name: string;
};
export type Files = {
    c: string[];
    cpp: string[];
    s: string[];
    inc: string[];
    paths: string[];
    a: string[];
};
export type Library = {
    rootpath: string;
    props: LibProps;
    files: Files;
};
export type Board = {
    symbols: SimpleSymbol;
    menuSelections: SimpleSymbol[];
};
export type BoardsList = {
    boards: Map<string, Board>;
    menus: Map<string, string>;
};
export type OldSizeType = {
    program: string;
    data: string;
    pattern: string;
    other: DumbSymTbl;
};
export type Pattern = {
    pattern: string;
    other: DumbSymTbl;
};
export type AllRecipes = {
    c: Pattern;
    cpp: Pattern;
    s: Pattern;
    ar: Pattern;
    link: Pattern;
    objcopy: {
        name: string;
        pattern: Pattern;
    }[];
    size: OldSizeType;
    advancedSize?: Pattern;
    preprocess?: Pattern;
    include?: Pattern;
    others: SimpleSymbol[];
};
export type AllHooks = {
    prebuild: Pattern[];
    postbuild: Pattern[];
    sketchPrebuild: Pattern[];
    skechPostbuild: Pattern[];
    libPrebuild: Pattern[];
    libPostbuild: Pattern[];
    corePrebuild: Pattern[];
    corePostbuild: Pattern[];
    prelink: Pattern[];
    postlink: Pattern[];
    precopy: Pattern[];
    postcopy: Pattern[];
    prehex: Pattern[];
    posthex: Pattern[];
};
export type Platform = {
    name: string;
    version: string;
    tools?: SimpleSymbol;
    misc: DumbSymTbl;
    recipes: AllRecipes;
    hooks: AllHooks;
    maxSize: {
        program: number;
        data: number;
    };
};
export type PlatformGlobalFuncs = {
    getRuntimePlatformPath: SFn;
    getRuntimeHardwarePath: SFn;
    getRuntimeIdePath: SFn;
    getRuntimeOs: SFn;
    getVendorName: SFn;
    getBoardId: SFn;
    getFQBN: SFn;
    getSourcePath: SFn;
    getLibDiscoveryPhase: SFn;
    getOptFlags: SFn;
    getTimeUtc: (tzAdjust?: boolean, dstAdjust?: boolean) => SFn;
};
export type BuildSystemHost = {
    emit: (platformPath: string, platform: Platform, board: BoardsList, libraries: Library[]) => Promise<void>;
    expandName: (nm: string) => {
        name: string;
        expansion: string;
    };
    globals: PlatformGlobalFuncs;
};
